
extern "C" {
#include <stdio.h>
#include "odicon.h"
}
#include <cmath>
#include "lbfgsb.h"

using namespace std;

#define ONE_SIDE_DIFF

static void gradient(int dim, double func(int dim, double x[]),
                     double x[], double g[]) {
    double xmax = 1.0, delta = 1.0e-5;
#ifdef ONE_SIDE_DIFF
    const double obj = func(dim, x);
#endif

    for (int i=0; i<dim; i++) {
        if (xmax < std::abs(x[i])) xmax = std::abs(x[i]);
    }
    delta *= xmax;

    for (int i=0; i<dim; i++) {
        double xsave = x[i];
#ifdef ONE_SIDE_DIFF
        x[i] -= delta;
        double fm = func(dim, x);  oracle_call++;
        g[i] = (fm - obj) / (x[i] - xsave);
#else
        double xm = x[i] = xsave - delta;
        double fm = func(dim, x);  oracle_call++;
        double xp = x[i] = xsave + delta;
        double fp = func(dim, x);  oracle_call++;
        g[i] = (fm - fp) / (xm - xp);
#endif
        x[i] = xsave;
    }
}


static double max_norm(int dim, double a[], double b[]) {
    double norm = 0;
    for (int i=0; i<dim; i++) {
        double tmp = std::abs(a[i] - b[i]);
        if (norm < tmp) norm = tmp;
    }
    return norm;
}

double lbfgs_call(const int dim, double func(int dim, double x[]), double x[]) {
    double *g, *xsave, obj, old_obj;
    LBFGS lbfgs;
    //const double delta = 1.0e-5;
    double registerd_xdelta = 0;
    int    registerd_iter   = 0;
    const double xdelta_threshold = 0.5;
    const int    iter_threshold   = 10;

    g     = new double[dim];
    xsave = new double[dim];
    counter_reset();
    obj = old_obj = func(dim, x);  oracle_call++;
    lbfgs.init(dim, 7); // # parameters, # LBFGS cache (useally 5 .. 7)
    for (;;) {
        memcpy(xsave, x, sizeof(*xsave) * dim);
        gradient(dim, func, x, g); // calc. gradient
        //printf("f = %g\n", func(dim, x));
        //printf("g = "); fvector_print(dim, g); printf("\n");
        //printf("x = "); fvector_print(dim, x); printf("\n");
        lbfgs.optimize(x, &old_obj, g); // call LBFGS

        registerd_xdelta += max_norm(dim, x, xsave);
        //printf("xdelta = %f, iter = %d\n", registerd_xdelta, registerd_iter);
        if (registerd_xdelta > xdelta_threshold) {
            registerd_xdelta = registerd_iter = 0;
        }
        if (++registerd_iter >= iter_threshold) break;

        obj = func(dim, x);  oracle_call++;   // obj eval.
        //if (std::abs((old_obj - obj) / obj) < delta) break;
        old_obj = obj;
        iteration++;
    }

#ifdef DEBUG
    printf("--------------------------\n");
    printf("[lbfgs]\n");
    printf("f(x) = %g, x = ", obj);
    fvector_print(dim, x);  printf("\n");
    counter_print();
#endif /* DEBUG */
    delete[] g;
    delete[] xsave;
    return obj;
}


double lbfgs_call_int(int dim, double func(int dim, double x[]), int x[]) {
    double *fx;

    fx = new double[dim];
    for (int i=0; i<dim; i++) fx[i] = x[i];
    double ret = lbfgs_call(dim, func, fx);
    for (int i=0; i<dim; i++) x[i] = (int)floor(0.5 + fx[i]);
    delete[] fx;
    return ret;
}
