
/* This is an M-convex function. */
/* laminar = sum(f(x)) */
/*   f(x) = ax^4 + bx^3 + cx^2 + dx + e  (type 4) */
/*   f(x) = ax^2 + bx + c                (type 2) */
/*   f(x) =  a * exp(bx + c)             (type E) */
/*   f(x) = -a * log(bx + c)             (type L) */
/*   f(x) =  a * abs(bx + c)             (type A) */
double flaminar(const int dim, double x[], double v[]);
double ilaminar(const int dim, int    x[], double v[]);

/* This is an L-convex function. */
/* pseparable = sum(f(xi-xj)) */
/*   f(x) is the same as laminar case */
double fpseparable(const int dim, double x[], double v[]);
double ipseparable(const int dim, int    x[], double v[]);

/********** a sample program ***********/
#if 0

#include <stdio.h>
#include <math.h>
#include "serialfunc.h"

void sample_for_flaminar(void) {
    double v[] = {
        'E', 2, 3, 4, /* 'E' needs three arguments */
        0, 1, 2, -1,  /* indexes of x[], -1 means end */
        '0',          /* end of equation */
    };
    double x[4] = { 9, 8, 7, 6 };
    double f = flaminar(4, x, v);
    
    printf("%f\n", f);
    printf("%f\n", 2*exp(3*(x[0]+x[1]+x[2])+4));
}


void sample_for_fpseparable(void) {
    double v[] = {
        'L', 2, 3, 4, /* 'L' needs three arguments */
        0, 1,         /* two indexes of x[], no end mark */
        '2', 5, 6, 7, /* '2' needs three arguments */
        3, -1,        /* one index of x[], -1 means ommited */
        '0',          /* end of equation */
    };
    double x[4] = { 9, 8, 7, 6 };
    double f = fpseparable(4, x, v);
    
    printf("%f\n", f);
    printf("%f\n", -2*log(3*(x[0]-x[1])+4) + 5*x[3]*x[3]+6*x[3]+7);
}

int main() {
    sample_for_flaminar();
    sample_for_fpseparable();
    return 0;
}
#endif
/********** a sample program ***********/
