#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "odicon.h"
#include "lseparable.h"

/* f1(x) = 2*x0^2 + 3*x1^2 - 2*x1*x2 */
/* quadratic ( Hessian = [2, -1; -1, 3] ) */
static double f1(int dim, int x[]) {
    double r = 0;

    assert(dim == 2);  /* check dimension */
    r +=  2 * x[0] * x[0];
    r +=  3 * x[1] * x[1];
    r += -2 * x[0] * x[1];
    return r;
}

static void input(int dim, int x[]) {
    int i;
    double tmp;
    char buff[1000];

    while (fgets(buff, sizeof buff, stdin) != NULL) {
        if (sscanf(buff, "%d,%lf", &i, &tmp) == 2) {
            if (0<=i && i<dim) x[i] = (int)floor(0.5 + tmp);
            else printf("out of range '%s'\n", buff);
        }
    }
}

struct lseparable_st *lseparable_inst;
static double ff2(int dim, double x[]) { return lseparable(dim, x, lseparable_inst); }
static double f2(int dim, int x[]) { return int_func(dim, ff2, x); }


static void test(int dim, double f(int,int*), int init[],
                 int lower[], int upper[]) {
    int i, x[1000];
    double ans;
    char *name[] = {"FIRST", "LAST",
                    "MINUS (murota)", "PLUS", "RANDOM", "KOLMOGOROV"};

    printf("--------------------------\n");
    printf("submodular (IFF)\n");
    ivector_copy(dim, x, init);
    ans = lgconv_minimize_IFF(dim, f, x);

    if (dim < 15) {
        ivector_copy(dim, x, init);
        printf("--------------------------\n");
        printf("steepest descent (minimizer:%s)\n", name[2]);
        ans = lgconv_minimize_type(dim, f, x, 2);
    }

    for (i=1; i<6; i++) {
        printf("--------------------------\n");
        printf("steepest descent (minimizer:%s)\n", name[i]);
        ivector_copy(dim, x, init);
        if ( lgconv_minimize_type(dim, f, x, i) != ans ) {
            printf("error\n"); exit(1);
        }
    }
}


void test_lconv(const int dim) {
    int i;
    int *init1 = ivector_new(2  /* dim */, 10, -10);
    int *init4 = ivector_new(dim /* dim */, 190, -200, 200);
    int *lower = ivector_new(10 /* dim */,-10000, -10000, -10000, -10000,
                             -10000, -10000, -10000, -10000, -10000, -10000);
    int *upper = ivector_new(10 /* dim */, 10000,  10000,  10000,  10000,
                             10000,  10000,  10000,  10000,  10000,  10000);

    lseparable_inst = lseparable_make(dim);

    test(2, f1, init1, lower,  upper);

    for (i=3; i<dim; i++) init4[i] = irandom(-10*dim, +10*dim);
    /*input(dim, init4);*/
    test(dim, f2, init4, lower, upper);
}
